@@ -531,7 +531,7 @@ async def test_broadcast(s, a, b):
     assert result == {a.address: b"pong", b.address: b"pong"}
 
 
-@gen_cluster(security=tls_only_security(),)
+@gen_cluster(security=tls_only_security())
 async def test_broadcast_tls(s, a, b):
     result = await s.broadcast(msg={"op": "ping"})
     assert result == {a.address: b"pong", b.address: b"pong"}
@@ -683,6 +683,8 @@ async def test_story(c, s, a, b):
 
     assert len(s.story(x.key, y.key)) > len(story)
 
+    assert s.story(x.key) == s.story(s.tasks[x.key])
+
 
 @gen_cluster(nthreads=[], client=True)
 async def test_scatter_no_workers(c, s):
@@ -965,7 +967,7 @@ async def test_worker_arrives_with_processing_data(c, s, a, b):
         await asyncio.sleep(0.01)
 
     w = Worker(s.address, nthreads=1)
-    w.put_key_in_memory(y.key, 3)
+    w.update_data(data={y.key: 3})
 
     await w
 
@@ -1017,7 +1019,7 @@ async def test_no_workers_to_memory(c, s):
         await asyncio.sleep(0.01)
 
     w = Worker(s.address, nthreads=1)
-    w.put_key_in_memory(y.key, 3)
+    w.update_data(data={y.key: 3})
 
     await w
 
@@ -1037,17 +1039,18 @@ async def test_no_workers_to_memory(c, s):
 
 @gen_cluster(client=True)
 async def test_no_worker_to_memory_restrictions(c, s, a, b):
-    x = delayed(slowinc)(1, delay=0.4)
-    y = delayed(slowinc)(x, delay=0.4)
-    z = delayed(slowinc)(y, delay=0.4)
+    with dask.annotate(workers="alice"):
+        x = delayed(slowinc)(1, delay=0.4)
+        y = delayed(slowinc)(x, delay=0.4)
+        z = delayed(slowinc)(y, delay=0.4)
 
-    yy, zz = c.persist([y, z], workers={(x, y, z): "alice"})
+    yy, zz = c.persist([y, z], optimize_graph=False)
 
     while not s.tasks:
         await asyncio.sleep(0.01)
 
     w = Worker(s.address, nthreads=1, name="alice")
-    w.put_key_in_memory(y.key, 3)
+    w.update_data(data={y.key: 3})
 
     await w
 
@@ -1127,7 +1130,7 @@ async def test_close_nanny(c, s, a, b):
         assert not a.is_alive()
         assert a.pid is None
 
-    while a.status != "closed":
+    while a.status != Status.closed:
         await asyncio.sleep(0.05)
         assert time() < start + 10
 
@@ -1136,7 +1139,7 @@ async def test_close_nanny(c, s, a, b):
 async def test_retire_workers_close(c, s, a, b):
     await s.retire_workers(close_workers=True)
     assert not s.workers
-    while a.status != "closed" and b.status != "closed":
+    while a.status != Status.closed and b.status != Status.closed:
         await asyncio.sleep(0.01)
 
 
@@ -1148,7 +1151,7 @@ async def test_retire_nannies_close(c, s, a, b):
 
     start = time()
 
-    while any(n.status != "closed" for n in nannies):
+    while any(n.status != Status.closed for n in nannies):
         await asyncio.sleep(0.05)
         assert time() < start + 10
 
@@ -1543,7 +1546,7 @@ async def test_closing_scheduler_closes_workers(s, a, b):
     await s.close()
 
     start = time()
-    while a.status != "closed" or b.status != "closed":
+    while a.status != Status.closed or b.status != Status.closed:
         await asyncio.sleep(0.01)
         assert time() < start + 2
 
@@ -1554,12 +1557,12 @@ async def test_closing_scheduler_closes_workers(s, a, b):
 async def test_resources_reset_after_cancelled_task(c, s, w):
     future = c.submit(sleep, 0.2, resources={"A": 1})
 
-    while not w.executing:
+    while not w.executing_count:
         await asyncio.sleep(0.01)
 
     await future.cancel()
 
-    while w.executing:
+    while w.executing_count:
         await asyncio.sleep(0.01)
 
     assert not s.workers[w.address].used_resources["A"]
@@ -1613,16 +1616,16 @@ async def test_idle_timeout(c, s, a, b):
     await future
     assert s.idle_since is None or s.idle_since > beginning
 
-    assert s.status != "closed"
+    assert s.status != Status.closed
 
     with captured_logger("distributed.scheduler") as logs:
         start = time()
-        while s.status != "closed":
+        while s.status != Status.closed:
             await asyncio.sleep(0.01)
             assert time() < start + 3
 
         start = time()
-        while not (a.status == "closed" and b.status == "closed"):
+        while not (a.status == Status.closed and b.status == Status.closed):
             await asyncio.sleep(0.01)
             assert time() < start + 1
 
@@ -1686,8 +1689,8 @@ async def test_result_type(c, s, a, b):
 @gen_cluster()
 async def test_close_workers(s, a, b):
     await s.close(close_workers=True)
-    assert a.status == "closed"
-    assert b.status == "closed"
+    assert a.status == Status.closed
+    assert b.status == Status.closed
 
 
 @pytest.mark.skipif(
@@ -1741,9 +1744,9 @@ async def test_adaptive_target(c, s, a, b):
 @pytest.mark.asyncio
 async def test_async_context_manager(cleanup):
     async with Scheduler(port=0) as s:
-        assert s.status == "running"
+        assert s.status == Status.running
         async with Worker(s.address) as w:
-            assert w.status == "running"
+            assert w.status == Status.running
             assert s.workers
         assert not s.workers
 
@@ -1918,8 +1921,7 @@ async def test_task_group_non_tuple_key(c, s, a, b):
 
 @gen_cluster(client=True)
 async def test_task_unique_groups(c, s, a, b):
-    """ This test ensure that task groups remain unique when using submit
-    """
+    """This test ensure that task groups remain unique when using submit"""
     x = c.submit(sum, [1, 2])
     y = c.submit(len, [1, 2])
     z = c.submit(sum, [3, 4])
@@ -1964,12 +1966,12 @@ class FlakyConnectionPool(ConnectionPool):
     def __init__(self, *args, failing_connections=0, **kwargs):
         self.cnn_count = 0
         self.failing_connections = failing_connections
-        super(FlakyConnectionPool, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     async def connect(self, *args, **kwargs):
         self.cnn_count += 1
         if self.cnn_count > self.failing_connections:
-            return await super(FlakyConnectionPool, self).connect(*args, **kwargs)
+            return await super().connect(*args, **kwargs)
         else:
             return BrokenComm()
 
@@ -2044,7 +2046,7 @@ async def test_gather_allow_worker_reconnect(c, s, a, b):
     s.rpc = await FlakyConnectionPool(failing_connections=4)
 
     with dask.config.set(
-        {"distributed.comm.retry.delay_min": 0.5, "distributed.comm.retry.count": 3,}
+        {"distributed.comm.retry.delay_min": 0.5, "distributed.comm.retry.count": 3}
     ):
         with captured_logger(
             logging.getLogger("distributed.scheduler")
@@ -2137,13 +2139,18 @@ async def test_worker_name_collision(s, a):
     # test that a name collision for workers produces the expected respsone
     # and leaves the data structures of Scheduler in a good state
     # is not updated by the second worker
-    with pytest.raises(ValueError, match=f"name taken, {a.name!r}"):
-        await Worker(s.address, name=a.name, loop=s.loop, host="127.0.0.1")
+    with captured_logger(logging.getLogger("distributed.scheduler")) as log:
+        with pytest.raises(ValueError, match=f"name taken, {a.name!r}"):
+            await Worker(s.address, name=a.name, loop=s.loop, host="127.0.0.1")
 
     s.validate_state()
     assert set(s.workers) == {a.address}
     assert s.aliases == {a.name: a.address}
 
+    log = log.getvalue()
+    assert "duplicate" in log
+    assert str(a.name) in log
+
 
 @gen_cluster(client=True, config={"distributed.scheduler.unknown-task-duration": "1h"})
 async def test_unknown_task_duration_config(client, s, a, b):
